from __future__ import division

import pygame

from pygame.locals import *

from math import sin, radians

import pickle

from effect import Fountain
from gui import *
from scene import *
from util import *

import map

TYPEA = {
    K_UP:(0,-1),
    K_DOWN:(0,1),
    K_LEFT:(-1,0),
    K_RIGHT:(1,0),

    K_w:(0,-1),
    K_s:(0,1),
    K_a:(-1,0),
    K_d:(1,0),
    
    K_KP8:(0,-1),
    K_KP2:(0,1),
    K_KP4:(-1,0),
    K_KP6:(1,0),
    K_KP7:(-1,-1),
    K_KP9:(1,-1),
    K_KP1:(-1,1),
    K_KP3:(1,1),
    }

TYPEB = {
    K_LEFT:(0,1),
    K_RIGHT:(0,-1),
    K_DOWN:(1,0),
    K_UP:(-1,0),

    K_a:(0,1),
    K_d:(0,-1),
    K_s:(1,0),
    K_w:(-1,0),

    K_KP8:(-1,0),
    K_KP2:(1,0),
    K_KP4:(0,1),
    K_KP6:(0,-1),
    K_KP7:(-1,1),
    K_KP9:(-1,-1),
    K_KP1:(1,1),
    K_KP3:(1,-1),
    }

colours = {
        "brown":Brown_Jelly,
        
        "red":Red_Jelly,
        "blue":Blue_Jelly,
        "yellow":Yellow_Jelly,
        
        "purple":Purple_Jelly,
        "orange":Orange_Jelly,
        "green":Green_Jelly,
        }

class Core:
    def __init__(self,screen):
        self.screen = screen
        self.numstars = 14
        self.save_data = {"stars":0,
                          "progress":0,
                          "levels":{"001":0,
                                    "002":0,
                                    "003":0,
                                    "004":0,
                                    "005":0,
                                    "006":0,
                                    "007":0,
                                    "008":0,
                                    "009":0,
                                    "010":0,
                                    },
                          }
        self.states = {
            "title":TitleState(self),
            "map":MapState(self),
            "battle":BattleState(self),
            }
        self.clock = pygame.time.Clock()
        self.c_state = self.states["title"]
    def change_state(self,state):
        self.c_state = self.states[state]
        if state == "map":
            self.c_state.reset()
        self.run()
    def begin_battle(self,level,diff):
        b =  self.states["battle"]
        b.load_level("%03d"%level,difficulty=diff)
        self.c_state = b
        self.run()
    def run(self):
        self.c_state.run()

class TitleState:
    def __init__(self,core):
        self.core = core
        self.screen = core.screen
        self.bg = load_image("title-bg")
        self.menu = MainMenu(self.screen)
    def draw(self):
        self.screen.blit(self.bg,(0,0))
        self.menu.draw()
        pygame.display.flip()
    def run(self):
        clock = self.core.clock
        while True:
            clock.tick(60)
            self.draw()
            event = pygame.event.poll()
            if event.type == QUIT:
                raise SystemExit()
            elif event.type == KEYDOWN:
                c = self.menu.key(event.key)
                if c == "Quit":
                    raise SystemExit()
                elif c == "New Game":
                    self.core.change_state("map")
                elif c == "Load Game":
                    f = open("savedata.txt")
                    data = pickle.load(f)
                    self.core.save_data = data
                    self.core.change_state("map")

class MapState:
    def __init__(self,core):
        self.core = core
        self.screen = self.core.screen
        self.arrow = {
            "easy":load_image("arrow-easy"),
            "medium":load_image("arrow-ally"),
            "hard":load_image("arrow-enemy"),
            }
        self.label = {
            "easy":load_image("label-easy"),
            "medium":load_image("label-medium"),
            "hard":load_image("label-hard"),
            }
        self.difficulties = ["easy","medium","hard","easy","medium","hard"]
        self.diff = 0
        self.numlevels = 2
        self.selected = 1
        self.time = 0
        self.reset()
        self.info = TextWin(self.screen,(300,380),self.levels[self.selected-1].desc,width = 300)
    def reset(self):
        f = open("savedata.txt","w")
        pickle.dump(self.core.save_data,f)
        self.stars = self.core.save_data["stars"]
        self.levels = []
        for i in range(self.numlevels):
            x = i*40
            y = sin(radians(i*40))*30
            completion = self.core.save_data["levels"]["%03d"%(i+1)]
            self.levels.append(LevelNode(self.screen,i+1,(x+100,y+250),completion))
        self.hud = LevelHud(self.screen,(100,400),self.core.save_data,self.core.numstars)
    def draw(self):
        self.screen.fill((0,0,0))
        x,y = self.levels[self.selected-1].pos
        cycle = sin(self.time/6)
        for l in self.levels:
            l.draw(self.stars)
        self.screen.blit(self.arrow[self.difficulties[self.diff]],(x+10,int(y-14+cycle*4)))
        self.screen.blit(self.label[self.difficulties[self.diff]],(x+20,y-14))
        self.info.draw()
        self.hud.draw(self.levels[self.selected-1])
        pygame.display.flip()
    def run(self):
        clock = self.core.clock
        while True:
            clock.tick(60)
            self.draw()
            self.time += 1
            event = pygame.event.poll()
            if event.type == QUIT:
                raise SystemExit()
            if event.type == KEYDOWN:
                if event.key == K_RIGHT:
                    self.selected = min(self.selected+1,self.numlevels)
                elif event.key == K_LEFT:
                    self.selected = max(self.selected-1,1)
                elif event.key in (K_x,K_RETURN,K_SPACE):
                    if self.levels[self.selected-1].cost <= self.stars:
                        if self.core.save_data["progress"]+1 >= self.selected:
                            self.core.begin_battle(self.selected,self.difficulties[self.diff])    
                elif event.key == K_UP and self.diff < 2:
                    completion = self.core.save_data["levels"]["%03d"%self.selected]
                    self.diff = min(self.diff + 1, completion)
                elif event.key == K_DOWN and self.diff > 0:
                    self.diff -= 1
                self.info.new_text(self.levels[self.selected-1].desc,width=300)

class BattleState:
    def __init__(self,core):
        self.core = core
        self.screen = core.screen
        self.type = TYPEA
        self.arrow = Arrow(self.screen)
        self.panel = StatView(self.screen)

        self.load_level("001")
        self.counter = Counter(self)
        self.help_win = TextWin(self.screen,(40,510),[
            "Welcome to Stratejelly. Hit the 'x' key over a spawn point to begin."
            ], width = 600)
        self.victory = load_image("victory")
        self.defeat = load_image("defeat")
        self.timeout = 180
    def load_level(self,level,difficulty = "easy"):
        self.reset()

        foes = {
            "grey":Grey_Ooze,
            }
        self.level = level
        diffs = {
            "easy":1,
            "medium":2,
            "hard":3
            }
        self.difficulty = diffs[difficulty]
        lv = open("data/levels/level-%s.txt"% level)
        L = pickle.load(lv)
        self.floor = map.load_map("map-%s"%level,self)
        self.jelly = L[difficulty][0]
        self.timeout = 180
        enemies = L[difficulty][1]
        for c,m,pos in enemies:
            s = self.floor.data[pos]
            unit = foes[c]((pos[0],pos[1],s.height()),m)
            s.add(unit)
            self.enemies.append(unit)
        self.camera.look_at(self.floor.cursor,25)
    def reset(self):
        self.camera = Camera(0,0)
        self.floor = None
        self.allies = []
        self.enemies = []
        self.effects = []
        self.phase = "free"
        self.selected = None
        self.highlights = {}
        self.undo = None
        self.side = "jelly"
        self.turn = 0
        self.jelly = 0
    def draw(self):
        self.screen.fill((0,0,0))
        self.floor.draw()
        self.arrow.draw(self.camera.offset())
        for fx in self.effects:
            if fx.dead:
                self.effects.remove(fx)
                continue
            fx.draw(self.screen,self.camera.offset())
        x,y = (self.floor.cursor.x,self.floor.cursor.y)
        u = None
        for t in self.floor.data[(x,y)].objects.__reversed__():
            if t.type == "unit":
                u = t
                break
        if u:
            self.panel.draw(u,(30,15))
        self.counter.draw((10,400))
        self.help_win.draw()
        if self.phase == "victory":
            self.screen.blit(self.victory,(375,200))
        elif self.phase == "defeat":
            self.screen.blit(self.defeat,(375,200))
        pygame.display.flip()
    def end_turn(self):
        self.selected = self.arrow.target = None
        self.turn += 1
        self.side = "jelly"
        self.floor.cursor.show(self.floor)
        self.phase = "free"
        for a in self.allies:
            a.tired = False
        for e in self.enemies:
            e.tired = False
        self.camera.look_at(self.floor.cursor,10)
    def run(self):
        clock = self.core.clock
        m = self.floor
        c = m.cursor
        targets = []
        mixing = False
        reagent = None
        while True:
            clock.tick(60)
            self.draw()
            if not self.enemies:
                self.phase = "victory"
            elif not self.allies and self.jelly < 10:
                self.phase = "defeat"
            event = pygame.event.poll()
            keys = pygame.key.get_pressed()
            if event.type == QUIT:
                raise SystemExit()

# FREE STATE
            if self.phase == "free":
                if self.allies:
                    exausted = True
                    for a in self.allies:
                        if not a.tired:
                            exausted = False                
                    if exausted:
                        self.help_win.new_text([
                            "Use the 't' key to end your turn.",
                            ],600)
                        self.help_win.visible = True
                    else:
                        self.help_win.visible = False
                else:
                    self.help_win.new_text([
                            "Welcome to Stratejelly. Hit the 'x' key over a spawn point to begin.",
                            ],600)
                    self.help_win.visible = True
                if event.type == KEYDOWN:
                    if event.key in (K_x,K_RETURN,K_SPACE,K_KP5):
                        o = c.select(m)
                        if o.type == "unit":
                            self.arrow.target = self.selected = o
                            if o.side == "jelly" and o.tired == False:
                                self.phase = "move"
                        elif o.type == "spawn":
                            s = self.floor.data[(c.x,c.y)]
                            unit = colours[o.colour]((c.x,c.y,s.height()))
                            if self.jelly >= unit.worth:
                                self.help_win.visible = False
                                self.jelly -= unit.worth
                                ux,uy = unit.pos
                                self.effects.append(FloatyNum("jelly-cube",-unit.worth,(ux,uy-64)))
                                s.add(unit)
                                self.allies.append(unit)
                                c.move((0,0),m)
                        else:
                            self.selected = self.arrow.target = None
                    elif event.key == K_t and len(self.allies) > 0:
                        c.hide(m)
                        self.phase = "ai"
                        self.side = "ooze"
                        continue
                for k in self.type:
                    if keys[k]:
                        c.move(self.type[k],m)
                        self.camera.look_at(c,8)

# MOVEMENT DECISION STATE
            elif self.phase == "move":
                # Find walkable area
                if not self.highlights:
                    self.highlights = self.selected.walkable_tiles(m)
                    # Add indicator panels
                    for x,y in self.highlights:
                        px,py = (self.selected.x,self.selected.y)
                        s = m.data[(x,y)]
                        if x != px or y != py:
                            if s.top().type == "unit":
                                s.add(Panel("purple",(x,y,s.height())))
                            else:
                                s.add(Panel("blue",(x,y,s.height())))
                        else:
                            s.add(Panel("wait",(x,y,s.height())))
                # Check for input
                for k in self.type:
                    if keys[k]:
                        c.move(self.type[k],m)
                        self.camera.look_at(c,8)
                    if event.type == KEYDOWN:
                        if event.key in (K_x,K_RETURN,K_SPACE,K_KP5):
                            x,y = (c.x,c.y)
                            if x == self.selected.x and y == self.selected.y:
                                for x,y in self.highlights:
                                    m.data[(x,y)].delight()
                                c.move((0,0),m)
                                self.highlights = {}
                                self.undo = (self.selected.x,self.selected.y,self.selected.z)
                                self.phase = "attack"
                            elif m.data[(x,y)].indicated():
                                if m.data[(x,y)].indicated("purple"):
                                    mixing = True
                                    reagent = m.data[(x,y)].top()
                                movement = self.highlights[(x,y)]
                                self.selected.path = movement[1:]
                                for x,y in self.highlights:
                                    m.data[(x,y)].delight()
                                c.move((0,0),m)
                                self.highlights = {}
                                self.undo = (self.selected.x,self.selected.y,self.selected.z)
                                self.phase = "moving"
                                c.hide(m)
                        elif event.key in (K_z,K_BACKSPACE):
                            for x,y in self.highlights:
                                m.data[(x,y)].delight()
                            c.move((0,0),m)
                            self.highlights = {}
                            self.arrow.target = self.selected = None
                            self.phase = "free"

# ENEMY AI STAGE
            elif self.phase == "ai":
                self.help_win.visible = False
                enemy = None
                for e in self.enemies:
                    if e.tired == False:
                        enemy = e
                        break
                if enemy:
                    self.selected = self.arrow.target = enemy
                    self.selected.ai(self)
                    self.phase = "moving"
                else:
                    self.end_turn()

# MOVEMENT STAGE
            elif self.phase == "moving":
                if self.selected.move(self):
                    self.camera.look_at(self.selected,16)
                else:
                    if mixing:
                        self.allies.remove(self.selected)
                        self.allies.remove(reagent)
                        self.allies.append(mix(self.selected,reagent,m))
                        reagent = None
                        mixing = False
                        self.selected = self.arrow.selected = None
                        self.phase = "free"
                    else:
                        if self.selected.tired:
                            self.phase = "free"
                            self.selected = self.arrow.selected = None
                        else:
                            self.phase = "attack"
                    c.show(m)

# ATTACK DECISION STATE
            elif self.phase == "attack":
                if self.side == "jelly":
                    # Find attackable squares
                    if not self.highlights:
                        self.highlights = self.selected.attack_tiles(m,(self.selected.x,self.selected.y))
                        # Add indicator panels
                        for x,y in self.highlights:
                            px,py = (self.selected.x,self.selected.y)
                            s = m.data[(x,y)]
                            if x != px or y != py:
                                s.add(Panel("red",(x,y,s.height())))
                            else:
                                s.add(Panel("wait",(x,y,s.height())))
                                
                    # Check for input
                    for k in self.type:
                        if keys[k]:
                            c.move(self.type[k],m)
                            self.camera.look_at(c,8)
                            # Remove yellow targets
                            for x,y in targets:
                                m.data[(x,y)].delight(colour="yellow")
                            # Set yellow targets
                            targets = self.selected.target_tiles(m,(c.x,c.y))
                            for x,y in targets:
                                s = m.data[(x,y)]
                                s.add(Panel("yellow",(x,y,s.height())))
                    if event.type == KEYDOWN:
                        if event.key in (K_x,K_RETURN,K_SPACE,K_KP5):
                            s = m.data[(c.x,c.y)]
                            if c.x == self.selected.x and c.y == self.selected.y:
                                for x,y in self.highlights:
                                    m.data[(x,y)].delight()
                                for x,y in targets:
                                    m.data[(x,y)].delight(colour="yellow")
                                targets = []
                                self.highlights = {}
                                self.phase = "free"
                                c.move((0,0),m)
                                self.selected.tired = True
                                self.arrow.target = self.selected = None
                            elif s.indicated("red"):
                                for x,y in self.highlights:
                                    m.data[(x,y)].delight()
                                for x,y in targets:
                                    m.data[(x,y)].delight(colour="yellow")
                                self.highlights = {}
                                self.phase = "combat"
                        if event.key in (K_z,K_BACKSPACE):
                            for x,y in self.highlights:
                                m.data[(x,y)].delight()
                            for x,y in targets:
                                m.data[(x,y)].delight(colour="yellow")
                            targets = []
                            self.highlights = {}
                            self.phase = "free"
                            self.selected.reset(self.undo,m)
                            self.camera.look_at(self.selected,10)
                            c.move((self.selected.x-c.x,self.selected.y-c.y),m)
                            self.arrow.target = self.selected = None
                elif self.side == "ooze":
                    targets = self.selected.target_tiles(m,self.selected.target)
                    self.phase = "combat"
# COMBAT STATE
            elif self.phase == "combat":
                if self.selected:
                    c.hide(m)
                    victims = find_targets(targets,m)
                    data = self.selected.action(targets,victims,(c.x,c.y),m)
                    for fx in data:
                        self.effects.append(fx)
                    a = self.selected
                    self.selected.tired = True
                    self.arrow.target = self.selected = None
                if self.effects == []:
                    for v in victims[:]:
                        d = v.hurt(a,self)
                        if d:
                            x,y = v.pos
                            self.effects.append(Fountain(v.colour,(x,y-32)))
                            m.data[(v.x,v.y)].remove(v)
                            victims.remove(v)
                            if v.side == "jelly":
                                self.allies.remove(v)
                            elif v.side == "ooze":
                                vx,vy = v.pos
                                self.effects.append(FloatyNum("jelly-cube",v.worth,(vx,vy-64)))
                                self.jelly += v.worth
                                self.enemies.remove(v)
                    victims = None
                    data = None
                    if self.side == "jelly":
                        self.phase = "free"
                        c.show(m)
                    else:
                        self.phase = "ai"
# VICTORY
            elif self.phase == "victory":
                self.timeout -= 1
                if self.timeout == 0:
                    completion = self.core.save_data["levels"][self.level]
                    if self.difficulty > completion:
                        self.core.save_data["levels"][self.level] = self.difficulty
                        self.core.save_data["stars"] += self.difficulty
                        self.core.save_data["progress"] = max(self.core.save_data["progress"],int(self.level))
                    self.core.change_state("map")
            elif self.phase == "defeat":
                self.timeout -= 1
                if self.timeout == 0:
                    self.core.change_state("map")

            self.camera.tick()
            m.cursor.tick()
