from __future__ import division

import pygame
from pygame.locals import *

import pickle

from math import radians, sin, cos

from effect import FloatyNum
from util import load_image

class MainMenu:
    def __init__(self,screen):
        self.screen = screen
        self.title = load_image("title")
        self.shade = load_image("menu-shade")
        self.wheel = load_image("colour-wheel")
        self.blue = load_image("jelly-blue-s")
        self.yellow = load_image("jelly-yellow-s")
        self.red = load_image("jelly-red-s")
        self.time = 0
        self.selected = 0
        self.angle = 270
        self.moving = False
        self.options = {
            0:"New Game",
            2:"Load Game",
            1:"Quit",
            }
        self.font = pygame.font.Font(None,32)
        self.info = TextWin(self.screen,(700,200),[
            "Keyboard Operated:",
            "Arrow keys: Menu control",
            "Space, Return, 'X': Confirm",
            "Backspace, 'Z': Cancel",
            "Arrow keys, WASD, Numpad: Move battle cursor",
            "'T': End turn"])
    def draw(self):
        self.time += 1
        cycle = sin(self.time/6)
        if self.angle%360 != self.selected*120+30:
            self.moving = True
            self.angle -= 3
        else:
            self.moving = False
        a = self.angle
        r = radians
        scale = pygame.transform.scale
        
        wheel = pygame.transform.rotate(self.wheel,-a)
        wrect = wheel.get_rect()
        wrect.center = (0,250)
        self.screen.blit(wheel,wrect)

        bimg = scale(self.blue,(int(64+cycle*7),int(64-cycle*10)))
        rimg = scale(self.red,(int(64+cycle*7),int(64-cycle*10)))
        yimg = scale(self.yellow,(int(64+cycle*7),int(64-cycle*10)))

        brect = bimg.get_rect()
        yrect = yimg.get_rect()
        rrect = rimg.get_rect()
        
        brect.center = (60*cos(r(a-30)),cycle*20+210+60*sin(r(a-30)))
        yrect.center = (60*cos(r(a+90)),cycle*20+210+60*sin(r(a+90)))
        rrect.center = (60*cos(r(a+210)),cycle*20+210+60*sin(r(a+210)))
        
        self.screen.blit(bimg,brect)
        self.screen.blit(yimg,yrect)
        self.screen.blit(rimg,rrect)

        self.screen.blit(self.title,(175,20))
        self.screen.blit(self.shade,(200,220))
        n = 0
        for i in [0,2,1]:
            if i == self.selected:
                if i == 0:
                    c = ((cycle+1)*80,(cycle+1)*80,(cycle+1)*120)
                elif i == 1:
                    c = ((cycle+1)*120,(cycle+1)*80,(cycle+1)*80)
                elif i == 2:
                    c = ((cycle+1)*120,(cycle+1)*120,(cycle+1)*80)
            else:
                c = (200,200,200)
            t = self.font.render(self.options[i],True,c)
            self.screen.blit(t,(230+n*40,250+n*50))
            n += 1

        self.info.draw()
    def key(self,key):
        if key in (K_UP,K_LEFT):
            self.selected = (self.selected+1)%3
        if key in (K_DOWN,K_RIGHT):
            self.selected = (self.selected-1)%3
        if key in (K_x,K_RETURN,K_SPACE):
            return self.options[self.selected]

class LevelNode:
    def __init__(self,screen,n,pos,completion):
        self.screen = screen
        self.number = n
        f = open("data/levels/level-%03d.txt"%n)
        data = pickle.load(f)
        self.name = data["name"]
        self.desc = [self.name]+data["desc"]
        self.cost = data["cost"]
        self.pos = pos
        self.done = completion
        self.complete = [load_image("level-c-easy"),
                         load_image("level-c-medium"),
                         load_image("level-c-hard")]
        self.incomplete = [load_image("level-i-easy"),
                         load_image("level-i-medium"),
                         load_image("level-i-hard")]
        self.price_tag = FloatyNum("star",self.cost,self.pos,life=-1)
    def draw(self,stars):
        d = self.done
        for c in self.complete[:d]:
            self.screen.blit(c,self.pos)
        if d < 3:
            self.screen.blit(self.incomplete[d],self.pos)
        if stars < self.cost:
            self.price_tag.draw(self.screen,(0,0))

class LevelHud:
    def __init__(self,screen,pos,data,tstars):
        self.screen = screen
        self.pos = pos
        self.labels = [load_image("label-easy"),load_image("label-medium"),load_image("label-hard")]
        self.star = load_image("star")
        self.empty = load_image("star-empty")
        self.font = pygame.font.Font(None,20)
        self.data = data
        self.tstars = tstars
    def draw(self,level):
        x,y = self.pos
        n = 0
        for L in self.labels:
            self.screen.blit(L,(x,y+n*25))
            for s in range(n+1):
                self.screen.blit(self.empty,(x+80+(18*s),8+y+n*25))
                if level.done > n:
                    self.screen.blit(self.star,(x+80+(18*s),8+y+n*25))
            n += 1
        t = self.font.render("Stars: %3d/%3d"%(self.data["stars"],self.tstars),True,(255,255,255))
        self.screen.blit(t,(x-30,y+100))
        P = (self.data["stars"]/self.tstars)*100
        c = self.font.render("Complete: %3d%%"%P,True,(255,255,255))
        self.screen.blit(c,(x-30,y+120))            

class StatView:
    def __init__(self,screen):
        self.screen = screen
        self.win = TextWin(screen,(0,0),["Textless"])
        self.bg = {
            "jelly":load_image("gui-jelly"),
            "ooze":load_image("gui-ooze"),
            }
        self.atk = load_image("gui-attack")
        self.heal = load_image("gui-heal")
        self.dfn = load_image("gui-defense")
        self.mov = load_image("gui-move")
        self.jmp = load_image("gui-jump")
        self.empty = load_image("gui-hbar-empty")
        self.bar = load_image("gui-hbar-full")

        self.weefont = pygame.font.Font(None,16)
        self.font = pygame.font.Font(None,18)
        self.namefont = pygame.font.Font(None,22)
    def draw(self,unit,pos):
        x,y = pos
        h,hp = (unit.max_hp,unit.hp)
        blit = self.screen.blit
        write = self.font.render
        script = self.weefont.render        

        blit(self.bg[unit.side],pos)
        blit(self.namefont.render(unit.name,True,(255,255,255)),(x+91,y+6))
        
        blit(self.mov,(x+98,y+26))
        blit(write("Mv. %3d"%unit.moverange,True,(255,255,255)),(x+122,y+32))

        if unit.attack >= 0:
            blit(self.atk,(x+98,y+51))
            blit(write("Ak. %3d"%unit.attack,True,(255,255,255)),(x+122,y+57))
        else:
            blit(self.heal,(x+98,y+51))
            blit(write("Hl. %3d"%-unit.attack,True,(180,255,180)),(x+122,y+57))
        
        blit(self.jmp,(x+173,y+26))
        blit(write("Jm. %3d"%unit.jump,True,(255,255,255)),(x+197,y+32))
        
        blit(self.dfn,(x+173,y+51))
        blit(write("Df. %3d"%unit.defense,True,(255,255,255)),(x+197,y+57))

        P = (float(hp)/h)
        blit(self.empty,(x+94,y+77))
        blit(self.bar,(x+115,y+91),(0,0,P*111,8))

        blit(script("HP %3d/%3d"%(unit.hp,unit.max_hp),True,(255,255,255)),(x+170,y+77))
        blit(script("%3d%%"%(P*100),True,(255,255,255)),(x+185,y+90))

        self.win.pos = (x,y+110)
        self.win.new_text(unit.desc,None)
        self.win.draw()

class Counter:
    def __init__(self,battle):
        self.battle = battle
        self.font = pygame.font.Font(None,16)
        self.jellycon = load_image("jelly-cubes")
        self.allycon = load_image("allies")
        self.enemycon = load_image("enemies")
        self.timecon = load_image("turns")
    def draw(self,pos):
        b = self.battle
        blit = self.battle.screen.blit
        write = self.font.render
        x,y = pos

        blit(self.jellycon,(x,y))
        blit(write("%4d"%b.jelly,True,(255,255,255)),(x+16,y+4))

        blit(self.allycon,(x,y+18))
        blit(write(str(len(b.allies)),True,(255,255,255)),(x+18,y+18+4))

        blit(self.enemycon,(x,y+36))
        blit(write(str(len(b.enemies)),True,(255,255,255)),(x+18,y+36+4))

        blit(self.timecon,(x,y+54))
        blit(write(str(b.turn),True,(255,255,255)),(x+18,y+54+4))

class TextWin:
    def __init__(self,screen,pos,text,visible = True,width = None):
        self.screen = screen
        self.pos = pos
        self.text = text
        self.font = pygame.font.Font(None,16)
        self.new_text(text,width)
        self.visible = visible
    def draw(self):
        if self.visible:
            x,y = self.pos
            w,h = (self.width,self.height)
            blit = self.screen.blit
            write = self.font.render
            drec = pygame.draw.rect

            drec(self.screen,(0,0,0),(x,y,w,h))
            drec(self.screen,(255,255,255),(x,y,w,h),1)
            drec(self.screen,(255,255,255),(x+2,y+2,w-4,h-4),1)
            
            n = 0
            for L in self.text:
                t = write(L,True,(255,255,255))
                blit(t,(x+4,n*16+y+7))
                n += 1
    def new_text(self,text,width):
        self.text = text
        self.height = 16*len(text)+8
        self.width = 0
        if width == None:
            for L in self.text:
                t = self.font.render(L,True,(255,255,255))
                w = t.get_width()
                if w > self.width:
                    self.width = w
        else:
            self.width = width
        self.width += 8
            
